@@ -16,6 +16,7 @@ import re
 import shutil
 import subprocess
 import tempfile
+from configparser import ConfigParser
 from pathlib import Path
 from typing import IO, Any, Dict, List, Optional
 
@@ -44,7 +45,7 @@ last_diagnostics: Dict[str, List[Dict[str, Any]]] = collections.defaultdict(list
 # Windows started opening opening a cmd-like window for every subprocess call
 # This flag prevents that.
 # This flag is new in python 3.7
-# THis flag only exists on Windows
+# This flag only exists on Windows
 windows_flag: Dict[str, int] = (
     {"creationflags": subprocess.CREATE_NO_WINDOW} if os.name == "nt" else {}  # type: ignore
 )
@@ -124,7 +125,7 @@ def pylsp_lint(
     config: Config, workspace: Workspace, document: Document, is_saved: bool
 ) -> List[Dict[str, Any]]:
     """
-    Lints.
+    Call the linter.
 
     Parameters
     ----------
@@ -145,20 +146,51 @@ def pylsp_lint(
     """
     settings = config.plugin_settings("pylsp_mypy")
     oldSettings1 = config.plugin_settings("mypy-ls")
-    if oldSettings1 != {}:
-        raise DeprecationWarning(
-            "Your configuration uses the namespace mypy-ls, this should be changed to pylsp_mypy"
-        )
     oldSettings2 = config.plugin_settings("mypy_ls")
-    if oldSettings2 != {}:
-        raise DeprecationWarning(
-            "Your configuration uses the namespace mypy_ls, this should be changed to pylsp_mypy"
+    if oldSettings1 != {} or oldSettings2 != {}:
+        raise NameError(
+            "Your configuration uses an old namespace (mypy-ls or mypy_ls)."
+            + "This should be changed to pylsp_mypy"
         )
     if settings == {}:
         settings = oldSettings1
         if settings == {}:
             settings = oldSettings2
 
+    if settings.get("report_progress", False):
+        with workspace.report_progress("lint: mypy"):
+            return get_diagnostics(config, workspace, document, settings, is_saved)
+    else:
+        return get_diagnostics(config, workspace, document, settings, is_saved)
+
+
+def get_diagnostics(
+    config: Config,
+    workspace: Workspace,
+    document: Document,
+    settings: Dict[str, Any],
+    is_saved: bool,
+) -> List[Dict[str, Any]]:
+    """
+    Lints.
+
+    Parameters
+    ----------
+    config : Config
+        The pylsp config.
+    workspace : Workspace
+        The pylsp workspace.
+    document : Document
+        The document to be linted.
+    is_saved : bool
+        Weather the document is saved.
+
+    Returns
+    -------
+    List[Dict[str, Any]]
+        List of the linting data.
+
+    """
     log.info(
         "lint settings = %s document.path = %s is_saved = %s",
         settings,
@@ -175,6 +207,9 @@ def pylsp_lint(
         log.warning("live_mode is not supported with dmypy, disabling")
         live_mode = False
 
+    if dmypy:
+        dmypy_status_file = settings.get("dmypy_status_file", ".dmypy.json")
+
     args = ["--show-column-numbers"]
 
     global tmpFile
@@ -207,6 +242,7 @@ def pylsp_lint(
         args.append("--strict")
 
     overrides = settings.get("overrides", [True])
+    exit_status = 0
 
     if not dmypy:
         args.extend(["--incremental", "--follow-imports", "silent"])
@@ -221,11 +257,12 @@ def pylsp_lint(
             )
             report = completed_process.stdout.decode()
             errors = completed_process.stderr.decode()
+            exit_status = completed_process.returncode
         else:
             # mypy does not exist on path, but must exist in the env pylsp-mypy is installed in
             # -> use mypy via api
             log.info("executing mypy args = %s via api", args)
-            report, errors, _ = mypy_api.run(args)
+            report, errors, exit_status = mypy_api.run(args)
     else:
         # If dmypy daemon is non-responsive calls to run will block.
         # Check daemon status, if non-zero daemon is dead or hung.
@@ -235,30 +272,39 @@ def pylsp_lint(
 
         if shutil.which("dmypy"):
             # dmypy exists on path
-            # -> use mypy on path
+            # -> use dmypy on path
             completed_process = subprocess.run(
-                ["dmypy", *apply_overrides(args, overrides)], stderr=subprocess.PIPE, **windows_flag
+                ["dmypy", "--status-file", dmypy_status_file, "status"],
+                stderr=subprocess.PIPE,
+                **windows_flag,
             )
-            _err = completed_process.stderr.decode()
-            _status = completed_process.returncode
-            if _status != 0:
+            errors = completed_process.stderr.decode()
+            exit_status = completed_process.returncode
+            if exit_status != 0:
                 log.info(
-                    "restarting dmypy from status: %s message: %s via path", _status, _err.strip()
+                    "restarting dmypy from status: %s message: %s via path",
+                    exit_status,
+                    errors.strip(),
+                )
+                subprocess.run(
+                    ["dmypy", "--status-file", dmypy_status_file, "restart"], **windows_flag
                 )
-                subprocess.run(["dmypy", "kill"], **windows_flag)
         else:
             # dmypy does not exist on path, but must exist in the env pylsp-mypy is installed in
             # -> use dmypy via api
-            _, _err, _status = mypy_api.run_dmypy(["status"])
-            if _status != 0:
+            _, errors, exit_status = mypy_api.run_dmypy(
+                ["--status-file", dmypy_status_file, "status"]
+            )
+            if exit_status != 0:
                 log.info(
-                    "restarting dmypy from status: %s message: %s via api", _status, _err.strip()
+                    "restarting dmypy from status: %s message: %s via api",
+                    exit_status,
+                    errors.strip(),
                 )
-                mypy_api.run_dmypy(["kill"])
+                mypy_api.run_dmypy(["--status-file", dmypy_status_file, "restart"])
 
         # run to use existing daemon or restart if required
-        args = ["run", "--"] + apply_overrides(args, overrides)
-
+        args = ["--status-file", dmypy_status_file, "run", "--"] + apply_overrides(args, overrides)
         if shutil.which("dmypy"):
             # dmypy exists on path
             # -> use mypy on path
@@ -268,16 +314,33 @@ def pylsp_lint(
             )
             report = completed_process.stdout.decode()
             errors = completed_process.stderr.decode()
+            exit_status = completed_process.returncode
         else:
             # dmypy does not exist on path, but must exist in the env pylsp-mypy is installed in
             # -> use dmypy via api
             log.info("dmypy run args = %s via api", args)
-            report, errors, _ = mypy_api.run_dmypy(args)
+            report, errors, exit_status = mypy_api.run_dmypy(args)
 
     log.debug("report:\n%s", report)
     log.debug("errors:\n%s", errors)
 
     diagnostics = []
+
+    # Expose generic mypy error on the first line.
+    if errors:
+        diagnostics.append(
+            {
+                "source": "mypy",
+                "range": {
+                    "start": {"line": 0, "character": 0},
+                    # Client is supposed to clip end column to line length.
+                    "end": {"line": 0, "character": 1000},
+                },
+                "message": errors,
+                "severity": 1 if exit_status != 0 else 2,  # Error if exited with error or warning.
+            }
+        )
+
     for line in report.splitlines():
         log.debug("parsing: line = %r", line)
         diag = parse_line(line, document)
@@ -329,7 +392,7 @@ def init(workspace: str) -> Dict[str, str]:
 
     configuration = {}
     path = findConfigFile(
-        workspace, ["pylsp-mypy.cfg", "mypy-ls.cfg", "mypy_ls.cfg", "pyproject.toml"]
+        workspace, [], ["pylsp-mypy.cfg", "mypy-ls.cfg", "mypy_ls.cfg", "pyproject.toml"], False
     )
     if path:
         if "pyproject.toml" in path:
@@ -338,14 +401,19 @@ def init(workspace: str) -> Dict[str, str]:
             with open(path) as file:
                 configuration = ast.literal_eval(file.read())
 
-    mypyConfigFile = findConfigFile(workspace, ["mypy.ini", ".mypy.ini", "pyproject.toml"])
+    configSubPaths = configuration.get("config_sub_paths", [])
+    mypyConfigFile = findConfigFile(
+        workspace, configSubPaths, ["mypy.ini", ".mypy.ini", "pyproject.toml", "setup.cfg"], True
+    )
     mypyConfigFileMap[workspace] = mypyConfigFile
 
     log.info("mypyConfigFile = %s configuration = %s", mypyConfigFile, configuration)
     return configuration
 
 
-def findConfigFile(path: str, names: List[str]) -> Optional[str]:
+def findConfigFile(
+    path: str, configSubPaths: List[str], names: List[str], mypy: bool
+) -> Optional[str]:
     """
     Search for a config file.
 
@@ -356,8 +424,12 @@ def findConfigFile(path: str, names: List[str]) -> Optional[str]:
     ----------
     path : str
         The path where the search starts.
+    configSubPaths : List[str]
+        Additional sub search paths in which mypy configs might be located
     names : List[str]
         The file to be found (or alternative names).
+    mypy : bool
+        whether the config file searched is for mypy (plugin otherwise)
 
     Returns
     -------
@@ -368,25 +440,38 @@ def findConfigFile(path: str, names: List[str]) -> Optional[str]:
     start = Path(path).joinpath(names[0])  # the join causes the parents to include path
     for parent in start.parents:
         for name in names:
-            file = parent.joinpath(name)
-            if file.is_file():
-                if file.name in ["mypy-ls.cfg", "mypy_ls.cfg"]:
-                    raise DeprecationWarning(
-                        f"{str(file)}: {file.name} is no longer supported, you should rename your "
-                        "config file to pylsp-mypy.cfg or preferably use a pyproject.toml instead."
-                    )
-                if file.name == "pyproject.toml":
-                    isPluginConfig = "pylsp-mypy.cfg" in names
-                    configPresent = (
-                        toml.load(file)
-                        .get("tool", {})
-                        .get("pylsp-mypy" if isPluginConfig else "mypy")
-                        is not None
-                    )
-                    if not configPresent:
-                        continue
-                return str(file)
-
+            for subPath in [""] + configSubPaths:
+                file = parent.joinpath(subPath).joinpath(name)
+                if file.is_file():
+                    if file.name in ["mypy-ls.cfg", "mypy_ls.cfg"]:
+                        raise NameError(
+                            f"{str(file)}: {file.name} is no longer supported, you should rename "
+                            "your config file to pylsp-mypy.cfg or preferably use a pyproject.toml "
+                            "instead."
+                        )
+                    if file.name == "pyproject.toml":
+                        configPresent = (
+                            toml.load(file).get("tool", {}).get("mypy" if mypy else "pylsp-mypy")
+                            is not None
+                        )
+                        if not configPresent:
+                            continue
+                    if file.name == "setup.cfg":
+                        config = ConfigParser()
+                        config.read(str(file))
+                        if "mypy" not in config:
+                            continue
+                    return str(file)
+    # No config file found in the whole directory tree
+    # -> check mypy default locations for mypy config
+    if mypy:
+        defaultPaths = ["~/.config/mypy/config", "~/.mypy.ini"]
+        XDG_CONFIG_HOME = os.environ.get("XDG_CONFIG_HOME")
+        if XDG_CONFIG_HOME:
+            defaultPaths.insert(0, f"{XDG_CONFIG_HOME}/mypy/config")
+        for path in defaultPaths:
+            if Path(path).expanduser().exists():
+                return str(Path(path).expanduser())
     return None
 
 
