@@ -1,5 +1,8 @@
 import collections
 import os
+import subprocess
+from pathlib import Path
+from typing import Dict
 from unittest.mock import Mock
 
 import pytest
@@ -9,15 +12,20 @@ from pylsp.workspace import Document, Workspace
 
 from pylsp_mypy import plugin
 
-DOC_URI = __file__
+# TODO using these file as a document is a bad idea as tests can break by adding new tests
+DOC_URI = f"file:/{Path(__file__)}"
 DOC_TYPE_ERR = """{}.append(3)
 """
-TYPE_ERR_MSG = '"Dict[<nothing>, <nothing>]" has no attribute "append"'
+TYPE_ERR_MSG = '"Dict[<nothing>, <nothing>]" has no attribute "append"  [attr-defined]'
 
 TEST_LINE = 'test_plugin.py:279:8: error: "Request" has no attribute "id"'
 TEST_LINE_WITHOUT_COL = "test_plugin.py:279: " 'error: "Request" has no attribute "id"'
 TEST_LINE_WITHOUT_LINE = "test_plugin.py: " 'error: "Request" has no attribute "id"'
 
+windows_flag: Dict[str, int] = (
+    {"creationflags": subprocess.CREATE_NO_WINDOW} if os.name == "nt" else {}  # type: ignore
+)
+
 
 @pytest.fixture
 def last_diagnostics_monkeypatch(monkeypatch):
@@ -36,7 +44,7 @@ def workspace(tmpdir):
 
 class FakeConfig(object):
     def __init__(self):
-        self._root_path = "C:" if os.name == "nt" else "/"
+        self._root_path = "C://" if os.name == "nt" else "/"
 
     def plugin_settings(self, plugin, document_path=None):
         return {}
@@ -62,8 +70,7 @@ def test_plugin(workspace, last_diagnostics_monkeypatch):
 
 
 def test_parse_full_line(workspace):
-    doc = Document(DOC_URI, workspace)
-    diag = plugin.parse_line(TEST_LINE, doc)
+    diag = plugin.parse_line(TEST_LINE)  # TODO parse a document here
     assert diag["message"] == '"Request" has no attribute "id"'
     assert diag["range"]["start"] == {"line": 278, "character": 7}
     assert diag["range"]["end"] == {"line": 278, "character": 8}
@@ -101,7 +108,7 @@ def foo():
     return
     unreachable = 1
 """
-    DOC_ERR_MSG = "Statement is unreachable"
+    DOC_ERR_MSG = "Statement is unreachable  [unreachable]"
 
     # Initialize two workspace folders.
     folder1 = tmpdir.mkdir("folder1")
@@ -140,6 +147,7 @@ def test_apply_overrides():
     assert plugin.apply_overrides(["1"], ["a", True, "b"]) == ["a", "1", "b"]
 
 
+@pytest.mark.skipif(os.name == "nt", reason="Not working on Windows due to test design.")
 def test_option_overrides(tmpdir, last_diagnostics_monkeypatch, workspace):
     import sys
     from stat import S_IRWXU
@@ -196,19 +204,84 @@ def test_option_overrides_dmypy(last_diagnostics_monkeypatch, workspace):
     m = Mock(wraps=lambda a, **_: Mock(returncode=0, **{"stdout.decode": lambda: ""}))
     last_diagnostics_monkeypatch.setattr(plugin.subprocess, "run", m)
 
+    document = Document(DOC_URI, workspace, DOC_TYPE_ERR)
+
     plugin.pylsp_lint(
         config=FakeConfig(),
         workspace=workspace,
-        document=Document(DOC_URI, workspace, DOC_TYPE_ERR),
+        document=document,
         is_saved=False,
     )
     expected = [
         "dmypy",
+        "--status-file",
+        ".dmypy.json",
         "run",
         "--",
         "--python-executable",
         "/tmp/fake",
         "--show-column-numbers",
-        __file__,
+        document.path,
     ]
-    m.assert_called_with(expected, stderr=-1, stdout=-1)
+    m.assert_called_with(expected, stderr=-1, stdout=-1, **windows_flag)
+
+
+def test_dmypy_status_file(tmpdir, last_diagnostics_monkeypatch, workspace):
+    statusFile = tmpdir / ".custom_dmypy_status_file.json"
+
+    last_diagnostics_monkeypatch.setattr(
+        FakeConfig,
+        "plugin_settings",
+        lambda _, p: {
+            "dmypy": True,
+            "live_mode": False,
+            "dmypy_status_file": str(statusFile),
+        }
+        if p == "pylsp_mypy"
+        else {},
+    )
+
+    document = Document(DOC_URI, workspace, DOC_TYPE_ERR)
+
+    assert not statusFile.exists()
+
+    plugin.pylsp_lint(
+        config=FakeConfig(),
+        workspace=workspace,
+        document=document,
+        is_saved=False,
+    )
+
+    assert statusFile.exists()
+
+
+def test_config_sub_paths(tmpdir, last_diagnostics_monkeypatch):
+    DOC_SOURCE = """
+def foo():
+    return
+    unreachable = 1
+"""
+    DOC_ERR_MSG = "Statement is unreachable  [unreachable]"
+
+    config_sub_paths = [".config"]
+
+    # Initialize workspace.
+    ws = Workspace(uris.from_fs_path(str(tmpdir)), Mock())
+    ws._config = Config(ws.root_uri, {}, 0, {})
+
+    # Create configuration file for workspace.
+    plugin_config = tmpdir.join("pyproject.toml")
+    plugin_config.write(f"[tool.pylsp-mypy]\nenabled = true\nconfig_sub_paths = {config_sub_paths}")
+    config_dir = tmpdir.mkdir(".config")
+    mypy_config = config_dir.join("mypy.ini")
+    mypy_config.write("[mypy]\nwarn_unreachable = True\ncheck_untyped_defs = True")
+
+    # Update settings for workspace.
+    plugin.pylsp_settings(ws._config)
+
+    # Test document to make sure it uses .config/mypy.ini configuration.
+    doc = Document(DOC_URI, ws, DOC_SOURCE)
+    diags = plugin.pylsp_lint(ws._config, ws, doc, is_saved=False)
+    assert len(diags) == 1
+    diag = diags[0]
+    assert diag["message"] == DOC_ERR_MSG
